# Generated by Django 5.1.7 on 2025-03-17 11:04

from django.db import migrations

import djangocms_link.fields


class Migration(migrations.Migration):

    dependencies = [
        ('djangocms_link', '0018_remove_link_anchor_remove_link_external_link_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='link',
            name='link',
            field=djangocms_link.fields.LinkField(default=dict, help_text='-', verbose_name='Link'),
        ),
    ]



from __future__ import annotations

from django.apps import apps
from django.contrib.sites.models import Site
from django.db import models


try:
    from filer.models import File
except (ModuleNotFoundError, ImportError):  # pragma: no cover
    File = None


def get_manager(model: models.Model, current_content: bool = False) -> models.Manager:
    if hasattr(model, "admin_manager"):
        return (
            model.admin_manager.current_content()
            if current_content
            else model.admin_manager
        )
    return model.objects


def get_rel_obj(internal_link: str) -> models.Model | None:
    if ":" in internal_link:
        model, pk = internal_link.split(":", 1)
        try:
            model = apps.get_model(*model.split(".", 1))
            return get_manager(model).filter(pk=pk).first()
        except (LookupError, ValueError):
            return None


def get_obj_link(obj: models.Model, site_id: int | None = None) -> str:
    # Access site id if possible (no db access necessary)
    if site_id is None:
        site_id = Site.objects.get_current().id
    obj_site_id = getattr(
        obj, "site_id", getattr(getattr(obj, "node", None), "site_id", None)
    )
    link = obj.get_absolute_url()  # Can be None
    if link and obj_site_id and obj_site_id != site_id:
        ref_site = Site.objects._get_site_by_id(obj_site_id).domain
        link = f"//{ref_site}{link}"
    return link


def get_link(link_field_value: dict, site_id: int | None = None) -> str | None:
    if not link_field_value:
        return None
    if "external_link" in link_field_value:
        if link_field_value["external_link"].startswith("tel:"):
            return link_field_value["external_link"].replace(" ", "")
        return link_field_value["external_link"] or None

    if "__cache__" in link_field_value:
        return link_field_value["__cache__"] or None

    if "internal_link" in link_field_value:
        obj = get_rel_obj(link_field_value["internal_link"])
    elif "file_link" in link_field_value:
        obj = get_rel_obj("filer.file:" + str(link_field_value["file_link"]))
    else:  # pragma: no cover
        return None

    if hasattr(obj, "get_absolute_url"):
        link_field_value["__cache__"] = get_obj_link(obj, site_id)  # Can be None
        if link_field_value["__cache__"]:
            link_field_value["__cache__"] += link_field_value.get("anchor", "")
    elif hasattr(obj, "url"):
        link_field_value["__cache__"] = obj.url
    else:
        link_field_value["__cache__"] = None
    return link_field_value["__cache__"]


class LinkDict(dict):
    """dict subclass with two additional properties: url and type to easily infer the link type and
    the url of the link. The url property is cached to avoid multiple db lookups."""

    def __init__(self, initial=None, **kwargs):
        anchor = kwargs.pop("anchor", None)
        super().__init__(**kwargs)
        if initial:
            if isinstance(initial, dict):
                self.update(initial)
            elif isinstance(initial, str):
                self["external_link"] = initial
            elif isinstance(initial, File):
                self["file_link"] = initial.pk
            elif isinstance(initial, models.Model):
                self["internal_link"] = (
                    f"{initial._meta.app_label}.{initial._meta.model_name}:{initial.pk}"
                )
                # Prepopulate cache since we have to object to get the URL
                self["__cache__"] = initial.get_absolute_url()
                if self["__cache__"] and anchor:
                    self["__cache__"] += anchor

    @property
    def url(self) -> str:
        return get_link(self) or ""

    @property
    def type(self) -> str:
        for key in ("internal_link", "file_link"):
            if key in self:
                return key
        if "external_link" in self:
            if self["external_link"].startswith("tel:"):
                return "tel"
            if self["external_link"].startswith("mailto:"):
                return "mailto"
            if self["external_link"].startswith("#"):
                return "anchor"
            return "external_link"
        return ""

    def __str__(self):
        """If inserted into a Django template, expand the url."""
        return self.url



from __future__ import annotations

import json

from django.apps import apps
from django.conf import settings
from django.contrib.admin import site
from django.contrib.admin.widgets import SELECT2_TRANSLATIONS, AutocompleteSelect
from django.contrib.sites.models import Site
from django.db import models
from django.db.models import JSONField, ManyToOneRel
from django.forms import Field, MultiWidget, Select, TextInput, URLInput
from django.utils.translation import get_language
from django.utils.translation import gettext_lazy as _

from cms.utils.urlutils import admin_reverse

from djangocms_link.helpers import LinkDict, get_manager


try:
    from filer.fields.file import AdminFileWidget, FilerFileField
    from filer.models import File
except (ModuleNotFoundError, ImportError):  # pragma: no cover
    File = None

from djangocms_link.validators import AnchorValidator, ExtendedURLValidator


MINIMUM_INPUT_LENGTH = getattr(settings, "DJANGOCMS_LINK_MINIMUM_INPUT_LENGTH", 0)


class LinkAutoCompleteWidget(AutocompleteSelect):
    def __init__(self, attrs: dict | None = None):
        super().__init__(None, None, attrs)

    def get_internal_obj(self, values: list[str | None]) -> list[models.Model | None]:
        internal_obj = []
        for value in values:
            if value:
                model_path, pk = value.split(":", 1)
                model = apps.get_model(*model_path.split(".", 1))
                internal_obj.append(get_manager(model).filter(pk=pk).first())
            else:
                internal_obj.append(None)
        return internal_obj

    def optgroups(self, name: str, value: str, attr: str | None = None):
        default = (None, [], 0)
        groups = [default]
        has_selected = False
        selected_choices = set(value)
        if not self.is_required and not self.allow_multiple_selected:
            default[1].append(self.create_option(name, "", "", False, 0))

        for option_value, option_label in zip(value, self.get_internal_obj(value)):
            selected = str(option_value) in value and (
                has_selected is False or self.allow_multiple_selected
            )
            has_selected |= selected
            index = len(default[1])
            subgroup = default[1]
            subgroup.append(
                self.create_option(
                    name, option_value, option_label, selected_choices, index
                )
            )
        return groups

    def get_url(self):
        return admin_reverse("djangocms_link_link_urls")

    def build_attrs(self, base_attrs: dict, extra_attrs: dict | None = None) -> dict:
        """
        Set select2's AJAX attributes.

        Attributes can be set using the html5 data attribute.
        Nested attributes require a double dash as per
        https://select2.org/configuration/data-attributes#nested-subkey-options
        """
        attrs = super(Select, self).build_attrs(base_attrs, extra_attrs=extra_attrs)
        attrs.setdefault("class", "")
        i18n_name = getattr(
            self, "i18n_name", SELECT2_TRANSLATIONS.get(get_language())
        )  # Django 3.2 compat
        attrs.update(
            {
                "data-ajax--cache": "true",
                "data-ajax--delay": 250,
                "data-ajax--type": "GET",
                "data-ajax--url": self.get_url(),
                "data-theme": "admin-autocomplete",
                "data-allow-clear": json.dumps(not self.is_required),
                "data-placeholder": "",  # Allows clearing of the input.
                "data-minimum-input-length": MINIMUM_INPUT_LENGTH,
                "lang": i18n_name,
                "class": attrs["class"]
                + (" " if attrs["class"] else "")
                + "admin-autocomplete",
            }
        )
        return attrs


class SiteAutocompleteSelect(AutocompleteSelect):
    no_sites = None

    def __init__(self, attrs: dict | None = None):
        # Hack: Pretend that the user is selecting a site for a Page object
        # and use Django admin's autocomplete widget
        try:
            from cms.models.pagemodel import TreeNode

            field = TreeNode._meta.get_field("site")
        except ImportError:  # pragma: no cover
            # django CMS 4.2+
            from cms.models import Page

            field = Page._meta.get_field("site")
        super().__init__(field, site, attrs)

    def optgroups(self, name: str, value: str, attr: dict | None = None):
        default = (None, [], 0)
        groups = [default]
        has_selected = False
        selected_choices = set(value)
        default[1].append(self.create_option(name, "", "", False, 0))

        site = Site.objects.get_current()
        option_value, option_label = site.pk, str(site)

        selected = str(option_value) in value and (
            has_selected is False or self.allow_multiple_selected
        )
        has_selected |= selected
        index = len(default[1])
        subgroup = default[1]
        subgroup.append(
            self.create_option(
                name, option_value, option_label, selected_choices, index
            )
        )
        return groups


# Configure the LinkWidget
link_types = {
    "internal_link": _("Internal link"),
    "external_link": _("External link/anchor"),
}
if File:
    link_types["file_link"] = _("File link")

# Get the allowed link types from the settings
allowed_link_types = getattr(
    settings,
    "DJANGOCMS_LINK_ALLOWED_LINK_TYPES",
    ("internal_link", "external_link", "file_link", "anchor", "mailto", "tel"),
)

# Adjust example uri schemes to allowed link types
example_uri_scheme = (
    "'https://'"
    + (", 'tel:'" if "tel" in allowed_link_types else "")
    + (", or 'mailto:'" if "mailto" in allowed_link_types else "")
)

# Show anchor sub-widget only for internal_link
_mapping = {key: key for key in link_types.keys()}
_mapping["anchor"] = "internal_link"

# Remove disallowed link types
link_types = {
    key: value for key, value in link_types.items() if key in allowed_link_types
}

# Create the available widgets
_available_widgets = {
    "always": Select(
        choices=list(link_types.items()),
        attrs={
            "class": "js-link-widget-selector",
            "data-help": _(
                "No destination selected. Use the dropdown to select a destination."
            ),
        },
    ),  # Link type selector
    "external_link": URLInput(
        attrs={
            "widget": "external_link",
            "placeholder": _("https://example.com or #anchor"),
            "data-help": _(
                "Provide a link to an external URL, including the schema such as {}. "
                "Optionally, add an #anchor (including the #) to scroll to."
            ).format(example_uri_scheme),
        },
    ),  # External link input
    "internal_link": LinkAutoCompleteWidget(
        attrs={
            "widget": "internal_link",
            "data-help": _(
                "Select from available internal destinations. Optionally, add an anchor to scroll to."
            ),
            "data-placeholder": _("Select internal destination"),
        },
    ),  # Internal link selector
    "anchor": TextInput(
        attrs={
            "widget": "anchor",
            "placeholder": _("#anchor"),
            "data-help": _("Provide an anchor to scroll to."),
        }
    ),
}
if File:
    _available_widgets["file_link"] = AdminFileWidget(
        rel=ManyToOneRel(FilerFileField, File, "id"),
        admin_site=site,
        attrs={
            "widget": "file_link",
            "data-help": _("Select a file as destination."),
        },
    )


class LinkWidget(MultiWidget):
    template_name = "djangocms_link/admin/link_widget.html"
    data_pos = {}
    number_sites = None
    default_site_selector = getattr(settings, "DJANGOCMS_LINK_SITE_SELECTOR", False)

    class Media:
        js = ("djangocms_link/link-widget.js",)
        css = {"all": ("djangocms_link/link-widget.css",)}

    def __init__(self, site_selector: bool | None = None):
        if site_selector is None:
            site_selector = LinkWidget.default_site_selector

        widgets = [
            widget
            for key, widget in _available_widgets.items()
            if key == "always" or _mapping[key] in link_types
        ]
        if site_selector and "internal_link" in allowed_link_types:
            index = next(
                i
                for i, widget in enumerate(widgets)
                if widget.attrs.get("widget") == "internal_link"
            )
            widgets.insert(
                index,
                SiteAutocompleteSelect(
                    attrs={
                        "class": "js-link-site-widget",
                        "widget": "site",
                        "data-placeholder": _("Select site"),
                    },
                ),
            )  # Site selector

        # Remember which widget expets its content at which position
        self.data_pos = {
            widget.attrs.get("widget"): i for i, widget in enumerate(widgets)
        }
        super().__init__(widgets)

    def get_context(self, name: str, value: str | None, attrs: dict) -> dict:
        if not self.is_required:
            self.widgets[0].choices = [("empty", "---------")] + self.widgets[0].choices
        context = super().get_context(name, value, attrs)
        context["widget"]["subwidgets"] = {
            widget["attrs"].get("widget", "link-type-selector"): widget
            for widget in context["widget"]["subwidgets"]
        }
        if File and "file_link" in allowed_link_types:
            del context["widget"]["subwidgets"]["file_link"]
            index = next(
                i
                for i, widget in enumerate(self.widgets)
                if widget.attrs.get("widget") == "file_link"
            )
            context["filer_widget"] = self.widgets[index].render(
                name + f"_{index}", value[index], attrs
            )
        return context


class LinkFormField(Field):
    widget = LinkWidget
    external_link_validators = [
        ExtendedURLValidator(allowed_link_types=allowed_link_types)
    ]
    internal_link_validators = []
    file_link_validators = []
    anchor_validators = [AnchorValidator()]

    empty_values = [None, {}]

    def __init__(self, *args, **kwargs):
        kwargs.setdefault("help_text", _("Select a link type and provide a link."))
        kwargs.setdefault("initial", {})
        kwargs.pop("encoder", None)  # Passed from LinkField's JSONField parent class
        kwargs.pop("decoder", None)  # but not needed
        super().__init__(*args, **kwargs)
        if isinstance(self.initial, dict):
            self.initial = self.prepare_value(self.initial)

    def prepare_value(self, value: dict) -> list[str | None]:
        if isinstance(value, list):
            return value
        multi_value = len(self.widget.widgets) * [None]
        if "external_link" in value:
            pos = self._get_pos("external_link")
            multi_value[0] = "external_link"
            multi_value[pos] = value["external_link"]
        elif "internal_link" in value:
            pos = self._get_pos("internal_link")
            anchor_pos = self._get_pos("anchor")
            multi_value[0] = "internal_link"
            multi_value[pos] = value["internal_link"]
            multi_value[anchor_pos] = value.get("anchor", "")
        elif "file_link" in value:
            multi_value[0] = "file_link"
            pos = self._get_pos("file_link")
            multi_value[pos] = str(value["file_link"])
        return multi_value

    def to_python(self, value: list[str | None]) -> dict:
        """Turn MultiWidget list data into LinkField dict format"""
        if not value:
            return LinkDict()

        link_type = value[0]
        pos = self._get_pos(link_type)
        if not pos or not value[pos]:  # No link type selected or no value
            return LinkDict()
        pos_anchor = self._get_pos("anchor")

        python = LinkDict({link_type: value[pos]} if value[pos] else {})
        if python and link_type == "internal_link" and pos_anchor and value[pos_anchor]:
            python["anchor"] = value[pos_anchor]
        return python

    def run_validators(self, value: LinkDict):
        """Check for <link_type>_validators property and run the validators"""
        for link_type in link_types:
            if link_type in value:
                self.validators = getattr(self, f"{link_type}_validators", [])
                super().run_validators(value[link_type])
        if "anchor" in value:
            self.validators = getattr(self, "anchor_validators", [])
            super().run_validators(value["anchor"])

    def _get_pos(self, link_type):
        """Returns the position of the different link type widgets"""
        return self.widget.data_pos.get(link_type)


class LinkField(JSONField):
    """A link is a JSON field with a default LinkFormField"""

    def __init__(self, *args, **kwargs):
        kwargs.setdefault("default", dict)
        kwargs.setdefault("help_text", "-")  # Help text is set by the widget
        super().__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        kwargs.setdefault("form_class", LinkFormField)
        return super().formfield(**kwargs)

    def get_prep_value(self, value):
        if isinstance(value, dict):
            # Drop any cached value without changing the original value
            return super().get_prep_value(dict(**{
                key: val for key, val in value.items() if key != "__cache__"
            }))
        return super().get_prep_value(value)

    def from_db_value(self, value, expression, connection):
        value = super().from_db_value(value, expression, connection)
        return LinkDict(value)

    def to_python(self, value):
        value = super().to_python(value)
        return LinkDict(value)



# Generated by Django 4.2.16 on 2024-10-24 19:59

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("djangocms_link", "0017_link_link"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="link",
            name="anchor",
        ),
        migrations.RemoveField(
            model_name="link",
            name="external_link",
        ),
        migrations.RemoveField(
            model_name="link",
            name="file_link",
        ),
        migrations.RemoveField(
            model_name="link",
            name="internal_link",
        ),
        migrations.RemoveField(
            model_name="link",
            name="mailto",
        ),
        migrations.RemoveField(
            model_name="link",
            name="phone",
        ),
    ]



from django import template
from django.db import models

from djangocms_link.helpers import LinkDict, get_link, get_obj_link


try:
    from filer.models import File
except (ImportError, ModuleNotFoundError):  # pragma: no cover

    class File:
        pass


register = template.Library()


@register.filter
def to_url(value):
    if isinstance(value, models.Model):
        return get_obj_link(value) or ""
    elif isinstance(value, dict):
        return get_link(value) or ""
    return ""


@register.filter
def to_link(value):
    return LinkDict(value)



from aldryn_client import forms


def split_and_strip(string):
    return [item.strip() for item in string.split(',') if item]


class Form(forms.BaseForm):
    templates = forms.CharField(
        'List of additional templates (comma separated)',
        required=False,
    )
    """
    The following settings need to be configured on your project separately
    as we don't want to expose them as aldryn configurations yet:
        DJANGOCMS_LINK_INTRANET_HOSTNAME_PATTERN
        DJANGOCMS_LINK_USE_SELECT2
    """

    def clean(self):
        data = super().clean()

        # prettify
        data['templates'] = ', '.join(split_and_strip(data['templates']))
        return data

    def to_settings(self, data, settings):
        if data['templates']:
            settings['DJANGOCMS_LINK_TEMPLATES'] = [
                (item, item)
                for item in split_and_strip(data['templates'])
            ]

        return settings



from __future__ import annotations

from django.apps import apps
from django.conf import settings
from django.contrib import admin
from django.core.exceptions import PermissionDenied
from django.db.models import F, Model, Prefetch, Q, QuerySet
from django.http import Http404, HttpRequest, JsonResponse
from django.urls import path
from django.utils.translation import gettext as _
from django.views.generic.list import BaseListView

from cms import __version__
from cms.models import Page
from cms.utils import get_language_from_request, get_language_list

from . import models
from .fields import LinkFormField, LinkWidget
from .helpers import get_manager


UNICODE_SPACE = "\u3000"  # This is a full-width space character (U+3000)


_version = int(__version__.split(".")[0])
if _version >= 4:
    from cms.admin.utils import GrouperModelAdmin
    from cms.models import PageContent
else:
    from cms.models import Title as PageContent

    class GrouperModelAdmin:
        pass


REGISTERED_ADMIN = []  # Will be set by djangocms_link.apps.DjangoCmsLinkConfig.ready


class AdminUrlsView(BaseListView):
    """Handle AutocompleteWidget's AJAX requests for data."""

    paginate_by = getattr(settings, "DJANGOCMS_LINK_PAGINATE_BY", 50)
    admin_site = None

    def get(self, request: HttpRequest, *args, **kwargs) -> JsonResponse:
        """
        Return a JsonResponse with search results (query parameter "q") usable by
        Django admin's autocomplete view. Each item is returned as defined in
        serialize_result(), for example:
        {
            "results": [
                {
                    "text": "Page",
                    "children" : [
                        {
                             "id": "cms.page:5",
                             "text": "My first page",
                             "url": "/en/my-first-page/",
                             "verbose_name": "Pages",
                         }, ...
                    ]
                 }, ...
             ],
             "pagination": {"more": true}
        }

        If the endpoint is called with the query parameter "g" (for get), the view will
        search for the id (e.g., "cms.page:5") and return its entry as defined in
        serialize_results(), e.g.:
        {
             "id": "cms.page:5",
             "text": "My first page",
             "url": "/en/my-first-page/",
             "verbose_name": "Pages",
         }
        """
        if request.GET.get("g"):
            # Get name of a reference
            return self.get_reference(request)

        self.term, self.language, self.site = self.process_request(request)

        if not self.has_perm(request):
            raise PermissionDenied

        qs_list = [self.get_queryset()]
        self.add_admin_querysets(qs_list)
        self.object_list = self.get_paginated_multi_qs(qs_list)
        context = self.get_context_data()
        results = self.get_optgroups(context)
        return JsonResponse(
            {
                "results": results,
                "pagination": {"more": context["page_obj"].has_next()},
            }
        )

    def get_page(self) -> int:
        page_kwarg = self.page_kwarg
        page = self.kwargs.get(page_kwarg) or self.request.GET.get(page_kwarg) or 1
        try:
            page_number = int(page)
        except ValueError:
            raise Http404(
                _("Page is not “last”, nor can it be converted to an int.")
            )
        return page_number

    def get_paginated_multi_qs(self, qs_list: list[QuerySet]) -> list[Model] | QuerySet:
        """
        Paginate multiple querysets and return a result list.
        """
        if len(qs_list) == 1:
            # Only one qs, just use regular pagination
            return qs_list[0]
        # Slize all querysets, evaluate and join them into a list
        max_items = self.get_page() * self.paginate_by
        objects = []
        for qs in qs_list:
            for item in qs:
                if self.has_perm(self.request, item):
                    objects.append(item)

            if len(objects) >= max_items:
                # No need to touch the rest of the querysets
                # as we have enough items already
                break
        return objects

    def get_reference(self, request: HttpRequest) -> JsonResponse:
        try:
            model_str, pk = request.GET.get("g").split(":")
            app, model = model_str.split(".")
            model = apps.get_model(app, model)
            model_admin = self.admin_site._registry.get(model)
            language = get_language_from_request(request)

            if model_str == "cms.page" and _version >= 4 or model_admin is None:
                obj = get_manager(model).get(pk=pk)
                if model_str == "cms.page":
                    obj.__link_text__ = obj.get_admin_content(language, fallback=True).title
                return JsonResponse(self.serialize_result(obj))
            elif model_str == "cms.page":
                obj = get_manager(model).get(pk=pk)
                obj.__link_text__ = obj.get_title(language, fallback=True)
                return JsonResponse(self.serialize_result(obj))

            if hasattr(model_admin, "get_link_queryset"):
                obj = model_admin.get_link_queryset(self.request, None).get(pk=pk)
            else:
                obj = model_admin.get_queryset(self.request).get(pk=pk)
            return JsonResponse(self.serialize_result(obj))
        except Exception as e:
            return JsonResponse({"error": str(e)})

    def get_optgroups(self, context):
        results = []
        model = {}
        previous_model = None
        for obj in context["object_list"]:
            if obj._meta.verbose_name_plural != previous_model or not model:
                if model:  # Don't add the initial empty model
                    results.append(model)
                previous_model = obj._meta.verbose_name_plural
                model = {
                    "text": previous_model.capitalize(),
                    "children": [],
                }
            model["children"].append(self.serialize_result(obj))
        if model:
            results.append(model)
        return results

    def serialize_result(self, obj: Model) -> dict:
        """
        Convert the provided model object to a dictionary that is added to the
        results list.
        """
        if isinstance(obj, Page) and hasattr(obj, "prefetched_content") and hasattr(obj, "get_admin_content"):
            obj.admin_content_cache = {trans.language: trans for trans in obj.prefetched_content}
            obj.__link_text__ = obj.get_admin_content(self.language).title

        indentation = UNICODE_SPACE * (max(getattr(obj, "__depth__", 1), 1) - 1)
        return {
            "id": f"{obj._meta.app_label}.{obj._meta.model_name}:{obj.pk}",
            "text": indentation + (getattr(obj, "__link_text__", str(obj)) or str(obj)),
            "url": obj.get_absolute_url(),
            "verbose_name": str(obj._meta.verbose_name).capitalize(),
        }

    def get_queryset(self) -> QuerySet:
        """Return queryset based on ModelAdmin.get_search_results()."""
        languages = get_language_list()
        try:
            # django CMS 4.1/5.0+
            content_qs = (
                PageContent.admin_manager.filter(language__in=languages)
                .filter(
                    Q(title__icontains=self.term) | Q(menu_title__icontains=self.term)
                )
                .current_content()
            )
            qs = (
                Page.objects.filter(pk__in=content_qs.values_list("page_id", flat=True))
                .order_by("path" if _version >= 5 else "node__path")
                .prefetch_related(
                    Prefetch(
                        "pagecontent_set",
                        to_attr="prefetched_content",
                        queryset=PageContent.admin_manager.current_content(),
                    ),
                )
            )
            if not self.term:
                qs = qs.annotate(
                    __depth__=F("depth" if _version >= 5 else "node__depth")
                )
            if self.site:
                qs = qs.filter(site_id=self.site) if _version >= 5 else qs.filter(node__site_id=self.site)
        except (AttributeError,):
            # django CMS 3.11
            qs = (
                get_manager(PageContent, current_content=True)
                .filter(language__in=languages)
                .filter(
                    Q(title__icontains=self.term) | Q(menu_title__icontains=self.term)
                )
            )
            qs = (
                Page.objects.filter(pk__in=qs.values_list("page_id", flat=True))
                .order_by("node__path")
                .prefetch_related(
                    Prefetch(
                        "title_set",
                        to_attr="prefetched_content",
                        queryset=get_manager(PageContent, current_content=True).all(),
                    ),
                )
            )
            if "publisher_draft" in Page._meta.fields_map:
                # django CMS 3.11
                qs = qs.filter(publisher_is_draft=True)
            if not self.term:
                qs = qs.annotate(
                    __depth__=F("node__depth")
                )

            if self.site:
                qs = qs.filter(node__site_id=self.site)
        return qs

    def add_admin_querysets(self, qs: list[QuerySet]) -> None:
        for model_admin in REGISTERED_ADMIN:
            try:
                # hack: GrouperModelAdmin expects a language to be temporarily set
                if isinstance(model_admin, GrouperModelAdmin):  # pragma: no cover
                    model_admin.language = self.language
                if hasattr(model_admin, "get_link_queryset"):
                    # Allow model admins to define get_link_queryset to do additional
                    # filtering, sorting and potentially custom site selection
                    new_qs = model_admin.get_link_queryset(self.request, self.site)
                else:
                    new_qs = model_admin.get_queryset(self.request)
                    if hasattr(model_admin.model, "site") and self.site:
                        new_qs = new_qs.filter(
                            Q(site_id=self.site) | Q(site__isnull=True)
                        )
                    elif hasattr(model_admin.model, "sites") and self.site:
                        new_qs = new_qs.filter(sites__id=self.site)
                new_qs, search_use_distinct = model_admin.get_search_results(self.request, new_qs, self.term)
                if search_use_distinct:  # pragma: no cover
                    new_qs = new_qs.distinct()
                qs.append(new_qs)
            except Exception:  # pragma: no cover
                # Still report back remaining urls even if one model fails
                pass

        return qs

    def process_request(self, request: HttpRequest) -> tuple[str, str, int | None]:
        """
        Validate request integrity, extract and return request parameters.
        """
        term = request.GET.get("term", request.GET.get("q", "")).strip("  ").lower()
        site = request.GET.get(
            "app_label", ""
        )  # Django admin's app_label is abused as site id
        try:
            site = int(site)
        except ValueError:
            site = None
        language = get_language_from_request(request)
        return term, language, site

    def has_perm(self, request: HttpRequest, obj=None) -> bool:
        """Check if user has permission to access the related model."""
        if obj is None:
            return True
        model_admin = self.admin_site._registry.get(obj.__class__)
        if model_admin is None:
            return False
        return model_admin.has_view_permission(request, obj=obj)


class LinkAdmin(admin.ModelAdmin):
    """The LinkAdmin class provides the endpoint for getting the urls. It is not visible in the
    admin interface."""

    global_link_form_widget = LinkWidget
    global_link_form_field = LinkFormField
    global_link_url_name = None

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.global_link_url_name = f"{self.opts.app_label}_{self.opts.model_name}_urls"

    def has_module_permission(self, request: HttpRequest) -> bool:  # pragma: no cover
        # Remove from admin
        return False

    def get_urls(self) -> list:
        # Only url endpoint public, do not call super().get_urls()
        return [
            path(
                "urls",
                self.admin_site.admin_view(self.url_view),
                name=self.global_link_url_name,
            ),
        ]

    def url_view(self, request: HttpRequest) -> JsonResponse:
        return AdminUrlsView.as_view(admin_site=self.admin_site)(request)


admin.site.register(models.Link, LinkAdmin)



#!/usr/bin/env python
import os
import sys

import django
from django.conf import global_settings, settings
from django.test.utils import get_runner

from tests.settings import HELPER_SETTINGS


CMS_APP = [
    "cms",
    "menus",
    "easy_thumbnails",
    "treebeard",
    "sekizai",
    "djangocms_link",
]
CMS_APP_STYLE = []
CMS_PROCESSORS = []
CMS_MIDDLEWARE = [
    "cms.middleware.user.CurrentUserMiddleware",
    "cms.middleware.page.CurrentPageMiddleware",
    "cms.middleware.toolbar.ToolbarMiddleware",
    "cms.middleware.language.LanguageCookieMiddleware",
]

INSTALLED_APPS = (
    [
        "django.contrib.contenttypes",
        "django.contrib.auth",
        "django.contrib.sessions",
        "django.contrib.sites",
        "django.contrib.staticfiles",
    ]
    + CMS_APP_STYLE
    + ["django.contrib.admin", "django.contrib.messages"]
    + CMS_APP
)
DATABASES = {"default": {"ENGINE": "django.db.backends.sqlite3", "NAME": ":memory:"}}
TEMPLATE_CONTEXT_PROCESSORS = [
    "django.template.context_processors.debug",
    "django.template.context_processors.request",
    "django.contrib.auth.context_processors.auth",
    "django.contrib.messages.context_processors.messages",
] + CMS_PROCESSORS
TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [
            os.path.join(os.path.dirname(__file__), "templates"),
            # insert your TEMPLATE_DIRS here
        ],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": TEMPLATE_CONTEXT_PROCESSORS,
        },
    },
]
MIDDLEWARE = [
    "django.middleware.http.ConditionalGetMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.middleware.locale.LocaleMiddleware",
    "django.middleware.common.CommonMiddleware",
] + CMS_MIDDLEWARE
SITE_ID = 1
LANGUAGE_CODE = "en"
LANGUAGES = (("en", "English"), ("fr", "French"))
CMS_LANGUAGES = {
    1: [
        {
            "code": "en",
            "name": "English",
            "public": True,
        },
        {
            "code": "fr",
            "name": "French",
            "public": True,
            "fallbacks": ["en"],
            "hide_untranslated": False,
        },
    ]
}
STATIC_URL = "/static/"
MEDIA_URL = "/media/"
MIGRATION_MODULES = {}
URL_CONF = "tests.utils.urls"


def pytest_configure():
    INSTALLED_APPS.extend(HELPER_SETTINGS.pop("INSTALLED_APPS"))

    settings.configure(
        default_settings=global_settings,
        **{
            **dict(
                INSTALLED_APPS=INSTALLED_APPS,
                TEMPLATES=TEMPLATES,
                DATABASES=DATABASES,
                SITE_ID=SITE_ID,
                LANGUAGE_CODE=LANGUAGE_CODE,
                LANGUAGES=LANGUAGES,
                MIGRATION_MODULES=MIGRATION_MODULES,
                ROOT_URLCONF=URL_CONF,
                STATIC_URL=STATIC_URL,
                MEDIA_URL=MEDIA_URL,
                SECRET_KEY="Secret!",
                MIDDLEWARE=MIDDLEWARE,
            ),
            **HELPER_SETTINGS,
        }
    )
    django.setup()


if __name__ == "__main__":
    pytest_configure()

    argv = ["tests"] if sys.argv is None else sys.argv
    tests = argv[1:] if len(argv) > 1 else ["tests"]
    TestRunner = get_runner(settings)
    test_runner = TestRunner()
    failures = test_runner.run_tests(tests)
    sys.exit(bool(failures))



# Generated by Django 4.2.16 on 2024-10-24 19:46

from django.db import migrations

import djangocms_link.fields


def forward(apps, schema_editor):
    Link = apps.get_model("djangocms_link", "Link")
    links = Link.objects.all()
    for link in links:
        if link.external_link:
            anchor = "#" + link.anchor if link.anchor else ""
            link.link = {"external_link": link.external_link + anchor}
        elif link.internal_link:
            opt = link.internal_link._meta
            link.link = {"internal_link": f"{opt.app_label}.{opt.model_name}:{link.internal_link.pk}"}
            if link.anchor:
                link.link["anchor"] = "#" + link.anchor
        elif link.file_link:
            link.link = {"file_link": str(link.file_link.pk)}
        elif link.phone:
            link.link = {"external_link": f"tel:{link.phone}"}
        elif link.mailto:
            link.link = {"external_link": f"mailto:{link.mailto}"}
        elif link.anchor:
            link.link = {"external_link": "#" + link.anchor}
    Link.objects.bulk_update(links, ["link"])


def backward(apps, schema_editor):
    raise NotImplementedError("Cannot reverse this migration.")


class Migration(migrations.Migration):
    dependencies = [
        ("djangocms_link", "0016_alter_link_cmsplugin_ptr"),
    ]

    operations = [
        migrations.AddField(
            model_name="link",
            name="link",
            field=djangocms_link.fields.LinkField(default={}, verbose_name="Link"),
        ),
        migrations.RunPython(forward, backward, elidable=True),
    ]



from django.apps import AppConfig, apps
from django.conf import settings
from django.contrib.admin import ModelAdmin
from django.core.exceptions import ImproperlyConfigured
from django.utils.translation import gettext_lazy as _


class DjangoCmsLinkConfig(AppConfig):
    name = "djangocms_link"
    verbose_name = _("django CMS Link")

    def ready(self):
        # Only scan admins after all apps are loaded
        from django.contrib import admin

        from djangocms_link import admin as link_admin

        linkable_models = getattr(settings, "DJANGOCMS_LINKABLE_MODELS", "auto")

        if linkable_models == "auto":  # pragma: no cover
            # Autoconfig? Check the admin registry for suitable admins
            link_admin.REGISTERED_ADMIN = []
            for _admin in admin.site._registry.values():
                if _admin.model._meta.app_label == "cms":
                    # Skip CMS models
                    continue
                # search_fields need to be defined in the ModelAdmin class, and the model needs to have
                # a get_absolute_url method.
                if getattr(_admin, "search_fields", []) and hasattr(
                    _admin.model, "get_absolute_url"
                ):
                    link_admin.REGISTERED_ADMIN.append(_admin)
        else:
            # turn model config into model admin instances
            admins = []
            for model in linkable_models:
                if isinstance(model, str):
                    model = apps.get_model(model)
                    if not hasattr(model, "get_absolute_url"):  # pragma: no cover
                        raise ImproperlyConfigured(
                            f"{model.__name__} needs to implement get_absolute_url method"
                        )
                    model_admin = admin.site._registry[model]
                    if model_admin not in admins:
                        admins.append(model_admin)
                elif not isinstance(model, ModelAdmin):  # pragma: no cover
                    raise ImproperlyConfigured(
                        'DJANGOCMS_LINK_LINKABLE_MODELS must be a list of string "app_label.model_name"'
                    )
            link_admin.REGISTERED_ADMIN = admins



from django.contrib.sites.shortcuts import get_current_site
from django.utils.translation import gettext_lazy as _

from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool

from .helpers import get_link
from .models import Link


class LinkPlugin(CMSPluginBase):
    model = Link
    name = _("Link")
    text_enabled = True
    text_icon = (
        '<svg xmlns="http://www.w3.org/2000/svg" width="16" height="16" fill="currentColor" class="bi bi-link-45deg" '
        'viewBox="0 0 16 16"><path d="M4.715 6.542 3.343 7.914a3 3 0 1 0 4.243 4.243l1.828-1.829A3 3 0 0 0 8.586 '
        "5.5L8 6.086a1 1 0 0 0-.154.199 2 2 0 0 1 .861 3.337L6.88 11.45a2 2 0 1 1-2.83-2.83l.793-.792a4 4 0 0 "
        '1-.128-1.287z"/><path d="M6.586 4.672A3 3 0 0 0 7.414 9.5l.775-.776a2 2 0 0 1-.896-3.346L9.12 3.55a2 2 0 '
        '1 1 2.83 2.83l-.793.792c.112.42.155.855.128 1.287l1.372-1.372a3 3 0 1 0-4.243-4.243z"/></svg>'
    )
    allow_children = True

    fieldsets = [
        (
            None,
            {
                "fields": (
                    "name",
                    "link",
                    "target",
                )
            },
        ),
        (
            _("Advanced settings"),
            {
                "classes": ("collapse",),
                "fields": (
                    "template",
                    "attributes",
                ),
            },
        ),
    ]

    def get_render_template(self, context, instance, placeholder):
        return f"djangocms_link/{instance.template}/link.html"

    def render(self, context, instance, placeholder):
        context["link"] = get_link(
            instance.link, getattr(get_current_site(context["request"]), "id", None)
        )
        return super().render(context, instance, placeholder)


plugin_pool.register_plugin(LinkPlugin)



"""
Enables the user to add a "Link" plugin that displays a link
using the HTML <a> tag.
"""

from django.conf import settings
from django.core.exceptions import ValidationError
from django.db import models
from django.utils.encoding import force_str
from django.utils.translation import gettext
from django.utils.translation import gettext_lazy as _

from cms.models import CMSPlugin

from djangocms_attributes_field.fields import AttributesField

from .fields import LinkField
from .helpers import get_link
from .validators import IntranetURLValidator


# Add additional choices through the ``settings.py``.
def get_templates():
    choices = [
        ("default", _("Default")),
    ]
    choices += getattr(
        settings,
        "DJANGOCMS_LINK_TEMPLATES",
        [],
    )
    return choices


HOSTNAME = getattr(settings, "DJANGOCMS_LINK_INTRANET_HOSTNAME_PATTERN", None)

TARGET_CHOICES = (
    ("_blank", _("Open in new window")),
    ("_self", _("Open in same window")),
    ("_parent", _("Delegate to parent")),
    ("_top", _("Delegate to top")),
)


class AbstractLink(CMSPlugin):
    # used by django CMS search
    search_fields = ("name",)

    # allows link requirement to be changed when another
    # CMS plugin inherits from AbstractLink
    link_is_optional = False

    url_validators = [
        IntranetURLValidator(intranet_host_re=HOSTNAME),
    ]

    template = models.CharField(
        verbose_name=_("Template"),
        choices=get_templates(),
        default=get_templates()[0][0],
        max_length=255,
    )
    name = models.CharField(
        verbose_name=_("Display name"),
        blank=True,
        max_length=255,
    )

    link = LinkField(
        verbose_name=_("Link"),
    )
    # advanced options
    target = models.CharField(
        verbose_name=_("Target"),
        choices=TARGET_CHOICES,
        blank=True,
        max_length=255,
    )
    attributes = AttributesField(
        verbose_name=_("Attributes"),
        blank=True,
        excluded_keys=["href", "target"],
    )

    # Add an app namespace to related_name to avoid field name clashes
    # with any other plugins that have a field with the same name as the
    # lowercase of the class name of this model.
    # https://github.com/divio/django-cms/issues/5030
    cmsplugin_ptr = models.OneToOneField(
        CMSPlugin,
        related_name="%(app_label)s_%(class)s",
        parent_link=True,
        on_delete=models.CASCADE,
    )

    class Meta:
        abstract = True

    def __str__(self):
        return self.name or str(self.pk)

    def get_short_description(self):
        link = self.get_link()
        if self.name and link:
            return f"{self.name} ({link})"
        return self.name or link or gettext("<link is missing>")

    def get_link(self, site_id=None):
        return get_link(self.link, site_id)

    def clean(self):
        super().clean()
        if not self.link_is_optional and not self.link:
            raise ValidationError(
                force_str(_("Link is required.")),
                code="required",
            )

    def __init__(self, *args, **wkargs):
        super().__init__(*args, **wkargs)
        self._meta.get_field("link").blank = self.link_is_optional


class Link(AbstractLink):
    class Meta:
        abstract = False



import re

from django.core.exceptions import ValidationError
from django.core.validators import EmailValidator, URLValidator
from django.utils.deconstruct import deconstructible
from django.utils.translation import gettext as _


class IntranetURLValidator(URLValidator):
    """
    This is essentially the normal, Django URL Validator, but allows for
    "internal" machine-name only "hostnames" as defined by the RegEx pattern
    defined in settings as well as normal, FQD-based hostnames.

    Some examples:
    RFC1123 Pattern
        DJANGOCMS_LINK_INTRANET_HOSTNAME_PATTERN = r'[a-z,0-9,-]{1,15}'
    """

    ul = "\u00a1-\uffff"  # unicode letters range (must be a unicode string, not a raw string)

    # IP patterns
    ipv4_re = (
        r"(?:25[0-5]|2[0-4]\d|[0-1]?\d?\d)(?:\.(?:25[0-5]|2[0-4]\d|[0-1]?\d?\d)){3}"
    )
    ipv6_re = r"\[[0-9a-f:\.]+\]"  # (simple regex, validated later)

    # Host patterns
    hostname_re = r"[a-z" + ul + r"0-9](?:[a-z" + ul + r"0-9-]*[a-z" + ul + r"0-9])?"
    domain_re = (
        r"(?:\.[a-z" + ul + r"0-9]+(?:[a-z" + ul + r"0-9-]*[a-z" + ul + r"0-9]+)*)*"
    )
    tld_re = r"\.[a-z" + ul + r"]{2,}\.?"
    host_re = "(" + hostname_re + domain_re + tld_re + "|localhost)"

    def __init__(self, intranet_host_re=None, **kwargs):
        super().__init__(**kwargs)
        if intranet_host_re:
            self.host_re = (
                "("
                + self.hostname_re
                + self.domain_re
                + self.tld_re
                + "|"
                + intranet_host_re
                + "|localhost)"
            )
            self.regex = re.compile(
                r"^(?:[a-z0-9\.\-]*)://"
                r"(?:\S+(?::\S*)?@)?"
                r"(?:" + self.ipv4_re + "|" + self.ipv6_re + "|" + self.host_re + ")"
                r"(?::\d{2,5})?"
                r"(?:[/?#][^\s]*)?"
                r"$",
                re.IGNORECASE,
            )


@deconstructible
class AnchorValidator:
    message = _("Enter a valid anchor")
    code = "invalid"

    def __call__(self, value: str):
        value = value.lstrip("#")
        if not value:
            return value
        if not isinstance(value, str) or len(value) > 100:
            raise ValidationError(self.message, code=self.code, params={"value": value})

        if not re.match(r"^[a-zA-Z0-9_\-]+$", value):
            raise ValidationError(self.message, code=self.code, params={"value": value})
        return value


class ExtendedURLValidator(IntranetURLValidator):
    # Phone numbers don't match the host regex in Django's validator,
    # so we test for a simple alternative.
    tel_re = r"^tel\:[0-9 \#\*\-\.\(\)\+]+$"

    def __init__(self, allowed_link_types: list = None, **kwargs):
        self.allowed_link_types = allowed_link_types
        super().__init__(**kwargs)

    def __call__(self, value: str):
        if not isinstance(value, str) or len(value) > self.max_length:
            raise ValidationError(self.message, code=self.code, params={"value": value})
        if self.unsafe_chars.intersection(value):
            raise ValidationError(self.message, code=self.code, params={"value": value})
        # Check if just an anchor
        if value.startswith("#") and (
            self.allowed_link_types is None or "anchor" in self.allowed_link_types
        ):
            return AnchorValidator()(value)
        # Check if the scheme is valid.
        scheme = value.split(":")[0].lower()
        if scheme == "tel" and (
            self.allowed_link_types is None or "tel" in self.allowed_link_types
        ):
            if re.match(self.tel_re, value):
                return
            else:
                raise ValidationError(
                    _("Enter a valid phone number"),
                    code=self.code,
                    params={"value": value},
                )
        if scheme == "mailto" and (
            self.allowed_link_types is None or "mailto" in self.allowed_link_types
        ):
            return EmailValidator()(value[7:])
        return super().__call__(value)
